<template>
  <div class="subpage_Box">
    <My-bread :list="[
      `${$t('dataManage.dataManage')}`,
      `知识图谱`,
    ]"></My-bread>
    <el-divider />
    <div class="mainBox">

      <div class="searchComp subpage_Div">
        <el-form ref="ruleForm" :model="ruleForm" :inline="true">
          <div class="flex_box">
            <div style="margin-right: auto">
              <el-form-item size="mini" :label="$t('dataManage.vmobj.keyword')" prop="name">
                <el-input v-model="ruleForm.name" :placeholder="$t('shuJuGuanLi.lable1')" style="width:200px"><i
                    slot="suffix" class="el-icon-search" @click="submitForm('ruleForm')"
                    style="padding-right: 8px"></i></el-input>
              </el-form-item>
            </div>
            <div>
              <el-form-item>
                <el-button icon="el-icon-refresh" type="info" size="mini">{{ $t('common.empty') }}
                </el-button>

              </el-form-item>
            </div>
          </div>

        </el-form>
      </div>
      <div class="dividing-line"></div>
      <div id="echartConent" class="echartBox">
        <div id="container" class="container" v-show="!showEcharts">
          <svg class="keywords" @mousemove="listener($event)">
            <a href="javascript:void(0)" v-for="(tag, index) in tags" :key="index" @click="nodeClick(tag)">
              <text class="text" :x="tag.x" :y="tag.y" :font-size="20 * (600 / (600 - tag.z))"
                :fill-opacity="((400 + tag.z) / 600)" :fill="tag.color">{{ tag.text }}
              </text>
            </a>
          </svg>

        </div>
        <div @click="setContentReturn" v-show="showEcharts">

          <el-link icon="el-icon-d-arrow-left"></el-link>
        </div>
        <div class="container" v-show="showEcharts">
          <div id="chartBox"></div>
        </div>
        <div id="menuTool" class="menuTool">
          <ul>
            <li @click="setMenuClick(1)">编辑</li>
            <li @click="setMenuClick(2)">删除</li>
          </ul>
        </div>
      </div>
    </div>
  </div>
</template>
<script>
import MyBread from "../../components/MyBread.vue";
import { knowledge_getGraph, knowledge_queryGraphResult } from "../../api/knowledge";
import * as echarts from "echarts"
import $ from 'jquery'
import { getToken } from '../../utils/auth';


export default {
  name: "catalogueManage",
  components: {
    MyBread,
  },
  data() {

    return {
      ruleForm: {},
      eData: {
        id: 70000,
        name: '周杰伦',
        categary: '百科',
        dataindex: 0,
        children: [{
          "id": 1599813927218,
          "name": "JayChou",
          "categary": "外文名",
          "dataindex": 1,
        }, {
          "id": 1599813927219,
          "name": "周董",
          "categary": "别名",
          "dataindex": 1,
        }, {
          "id": 1599813927220,
          "name": "中国",
          "categary": "国籍",
          "dataindex": 1,
        }, {
          "id": 1599813927221,
          "name": "汉族",
          "categary": "民族",
          "dataindex": 1,
        }, {
          "id": 1599813927222,
          "name": "摩羯座",
          "categary": "星座",
          "dataindex": 1,
        }, {
          "id": 1599813927223,
          "name": "O型",
          "categary": "血型",
          "dataindex": 1,
        }, {
          "id": 1599813927224,
          "name": "175cm",
          "categary": "身高",
          "dataindex": 1,
        }, {
          "id": 1599813927225,
          "name": "台湾省新北市",
          "categary": "出生地",
          "dataindex": 1,
        }]
      },

      tags: [],
      RADIUS: 200,
      ZRADIUS: 200,
      CX: 600,
      CY: 300,
      speedX: Math.PI / 360,
      speedY: Math.PI / 360,
      showEcharts: false,
      myChart: null,
      seriesData: [],
      seriesLinks: [],
    };
  },
  watch: {

  },
  methods: {
    submitForm() {

    },
    setContentReturn() {
      this.ruleForm.name = null;
      this.showEcharts = false;
    },
    setMenuClick(res) {
      console.log("4", res)
    },
    async nodeClick(tag) {

      const data = await knowledge_queryGraphResult({
        domain: tag.text,
        nodeName: "",
        pageSize: 500,
      })
      if (data.status != 200) return
      console.log("Tag", data)
    
      this.showEcharts = true;
      // this.seriesData.push(...data.data.data.node)
      // this.seriesLinks.push(...data.data.data.relationship)

      // this.initEcharts()
      this.formatData([this.eData] || [], true)
    },
    formatData(list, reset = false) {
      let nodes = []
      const data = []
      const links = []
      let target = ''
      if (reset) {
        this.seriesData = []
        this.seriesLinks = []
        nodes = [].concat(list[0].children)
        target = list[0].id + ''
        const dataInfo = {
          "id": target,
          "name": list[0].name,
          category: list[0].categary,
          isClicked: true,
          isRoot: true,
          symbolSize: 30,
          dataindex: list[0].dataindex
        }
        data.push(dataInfo)
      } else {
        nodes = [].concat(list)
        target = this.lastClickId
      }

      nodes.forEach((item, index) => {
        // 注意 id必须是string  否则连线连不上
        const id = item.id + ''
        const dataInfo = {
          "id": id,
          parentId: target,
          category: item.categary,
          "name": item.name,
          isClicked: false,
          dataindex: item.dataindex
        }
        data.push(dataInfo)
        if (target != item.id) {
          links.push({
            value: item.categary,
            target: target,
            source: id,
          })
        }

      })
      console.log(data,links)
      this.seriesData.push(...data)
      this.seriesLinks.push(...links)

      this.initEcharts()
    },
    initEcharts() {
      document.getElementById("chartBox").style.width = document.getElementById("echartConent").offsetWidth + "px";
      document.getElementById("chartBox").style.height = document.getElementById("echartConent").offsetHeight + "px";
      this.myChart = echarts.init(document.getElementById('chartBox'));
      let option = {
        // 动画更新变化时间
        animationDurationUpdate: 1500,
        animationEasingUpdate: 'quinticInOut',
        tooltip: {
          show: false
        },
        series: [
          {
            type: 'graph',
            layout: 'force',
            legendHoverLink: true, //是否启用图例 hover(悬停) 时的联动高亮。
            hoverAnimation: true, //是否开启鼠标悬停节点的显示动画
            edgeLabel: {
              position: 'middle',       //边上的文字样式
              normal: {
                formatter: "{c}",
                show: true,

              }
            },
            edgeSymbol: ['arrow', ''],
            force: {
              edgeLength: 120,
              repulsion: 500
            },
            roam: true,
            draggable: true,//每个节点的拖拉
            itemStyle: {
              normal: {
                color: function (param) {
                  var colorList = ['red', 'orange', 'green']
                  return colorList[param.data.dataindex]
                },
                cursor: 'pointer',
                //color:Math.floor(Math.random()*16777215).toString(16),
                // color: ['#fc853e','#28cad8','#9564bf','#bd407e','#28cad8','#fc853e','#e5a214'],
                label: {
                  show: true,
                  position: [-10, -15],
                  textStyle: { //标签的字体样式
                    color: 'BLACK', //字体颜色
                    fontStyle: 'normal',//文字字体的风格 'normal'标准 'italic'斜体 'oblique' 倾斜
                    fontWeight: 'bolder',//'normal'标准'bold'粗的'bolder'更粗的'lighter'更细的或100 | 200 | 300 | 400...
                    fontFamily: 'sans-serif', //文字的字体系列
                    fontSize: 12, //字体大小
                  }
                },
                nodeStyle: {
                  brushType: "both",
                  borderColor: "rgba(255,215,0,0.4)",
                  borderWidth: 1,
                },
              },
              //鼠标放上去有阴影效果
              emphasis: {
                shadowColor: '#00FAE1',
                shadowOffsetX: 0,
                shadowOffsetY: 0,
                shadowBlur: 40,
              },
            },
            lineStyle: {
              width: 2,
            },
            label: {
              color: '#00FAE1',
              fontSize: 18,
            },
            symbolSize: 14,//节点大小
            links: this.seriesLinks,
            data: this.seriesData,
            cursor: 'pointer',
          }
        ]
      };
      // 使用刚指定的配置项和数据显示图表。
      this.myChart.setOption(option);
      if (this.myChart) {
        // 隐藏右键保存图片
        var tree = document.getElementById("echartConent");
        tree.oncontextmenu = function () {
          return false;
        };
        // 点击事件
        this.myChart.on("click", function (param) {
          if (param.data) {
            $("#menuTool").hide();
          }
        })
        document.onclick = function () {
          $("#menuTool").hide();
        }
        document.oncontextmenu = function (param)  //绑定右键
        {
          document.getElementById("menuTool").style.left = param.clientX + 15 + "px";
          document.getElementById("menuTool").style.top = param.clientY - 15 + "px";
        }
        // 自定义右键事件
        this.myChart.on("contextmenu", function (param) {
          if (param.data) {
            $("#menuTool").show();
          }
        })


      }
    },
    async initTags() {
      const data = await knowledge_getGraph({});
      if (data.status != 200) return;
      const tagNames = data.data.data.nodeList;
      //初始化标签位置
      let tags = [];
      const length = tagNames.length
      for (let i = 0; i < length; i++) {
        let tag = {};
        let k = -1 + (2 * (i + 1) - 1) / length;
        let a = Math.acos(k);
        let b = a * Math.sqrt(length * Math.PI);
        tag.text = tagNames[i].label;
        tag.id = tagNames[i].id
        tag.x = this.CX + this.RADIUS * Math.sin(a) * Math.cos(b);
        tag.y = this.CY + this.RADIUS * Math.sin(a) * Math.sin(b);
        tag.z = this.ZRADIUS * Math.cos(a);
        tag.color =
          "rgb(" +
          parseInt(Math.random() * 255) +
          "," +
          parseInt(Math.random() * 255) +
          "," +
          parseInt(Math.random() * 255) +
          ")";
        tags.push(tag);
      }
      this.tags = [].concat(tags);
    },
    rotateX(speedX) {
      var cos = Math.cos(speedX);
      var sin = Math.sin(speedX);
      for (let tag of this.tags) {
        var y1 = (tag.y - this.CY) * cos - tag.z * sin + this.CY;
        var z1 = tag.z * cos + (tag.y - this.CY) * sin;
        tag.y = y1;
        tag.z = z1;
      }
    },
    /**
     * 自动滚动效果,计算滚动位置
     */
    rotateY(speedY) {
      var cos = Math.cos(speedY);
      var sin = Math.sin(speedY);
      for (let tag of this.tags) {
        var x1 = (tag.x - this.CX) * cos - tag.z * sin + this.CX;
        var z1 = tag.z * cos + (tag.x - this.CX) * sin;
        tag.x = x1;
        tag.z = z1;
      }
    },
    /**
     * 响应鼠标移动
     */
    listener(event) {
      //
      var x = event.clientX - this.CX;
      var y = event.clientY - this.CY;
      this.speedX =
        x * 0.0001 > 0
          ? Math.min(this.RADIUS * 0.00002, x * 0.0001)
          : Math.max(-this.RADIUS * 0.00002, x * 0.0001);
      this.speedY =
        y * 0.0001 > 0
          ? Math.min(this.RADIUS * 0.00002, y * 0.0001)
          : Math.max(-this.RADIUS * 0.00002, y * 0.0001);
    },
    /**
     * 监听窗体大小变化
     */
    resizeWindow() {
      if (this.showEcharts) return
      let height = document.getElementById("container").clientHeight
      let width = document.getElementById("container").clientWidth
      width = width * 0.85;
      if (width > 1200) {
        this.CX = width / 2
      }
      height = height - 150
      this.CY = height / 2;
      let radius = Math.min(this.CY, this.CX) / 2
      if (radius > 200) {
        this.RADIUS = radius
      }
      this.initTags()

    },
    setResizeStart() {
      this.resizeWindow()
      window.addEventListener('resize', this.resizeWindow)
      this.initTags();
      //使球开始旋转
      const interval = setInterval(() => {
        this.rotateX(this.speedX);
        this.rotateY(this.speedY);
      }, 17);
      this.$once('hook:beforedestroy', () => {
        interval && clearInterval(interval)
        window.removeEventListener('resize', this.resizeWindow)
      })
    },

  },
  mounted() {
    this.setResizeStart();
  },
};
</script>

<style lang="less" scoped>
.subpage_Box {
  width: 98%;
  padding: 1%;
  border-radius: 10px;
}

.mainBox {
  padding-top: 10px;
  width: 100%;
  height: calc(100% - 75px);

  .echartBox {
    width: 100%;
    height: calc(94% - 10px);
    margin-top: 10px;
    z-index: 99;

    .container,
    .keywords {
      width: 100%;
      height: 100%;
    }

    .keywords .text:hover {
      font-size: 200%;
    }


  }

  .menuTool {
    position: absolute;
    z-index: 99999999999;

    border: 1px solid gray;
    background: white;
    border-radius: 5%;
    display: none;
    padding: 0px 10px;

    li {
      line-height: 20px;
      font-size: 12px;
    }

    li:hover {
      color: #409EFF;
    }
  }

}
</style>
